@@ -58,11 +58,16 @@ struct cmatest_node {
 	struct rdma_cm_id	*cma_id;
 	int			connected;
 	struct ibv_pd		*pd;
-	struct ibv_cq		*cq;
+	struct ibv_cq		*cq[2];
 	struct ibv_mr		*mr;
 	void			*mem;
 };
 
+enum CQ_INDEX {
+	SEND_CQ_INDEX,
+	RECV_CQ_INDEX
+};
+
 struct cmatest {
 	struct rdma_event_channel *channel;
 	struct cmatest_node	*nodes;
@@ -124,27 +129,28 @@ static int init_node(struct cmatest_node *node)
 		goto out;
 	}
 
-	cqe = message_count ? message_count * 2 : 2;
-	node->cq = ibv_create_cq(node->cma_id->verbs, cqe, node, 0, 0);
-	if (!node->cq) {
+	cqe = message_count ? message_count : 1;
+	node->cq[SEND_CQ_INDEX] = ibv_create_cq(node->cma_id->verbs, cqe, node, 0, 0);
+	node->cq[RECV_CQ_INDEX] = ibv_create_cq(node->cma_id->verbs, cqe, node, 0, 0);
+	if (!node->cq[SEND_CQ_INDEX] || !node->cq[RECV_CQ_INDEX]) {
 		ret = -ENOMEM;
 		printf("cmatose: unable to create CQ\n");
 		goto out;
 	}
 
 	memset(&init_qp_attr, 0, sizeof init_qp_attr);
-	init_qp_attr.cap.max_send_wr = message_count ? message_count : 1;
-	init_qp_attr.cap.max_recv_wr = message_count ? message_count : 1;
+	init_qp_attr.cap.max_send_wr = cqe;
+	init_qp_attr.cap.max_recv_wr = cqe;
 	init_qp_attr.cap.max_send_sge = 1;
 	init_qp_attr.cap.max_recv_sge = 1;
 	init_qp_attr.qp_context = node;
 	init_qp_attr.sq_sig_all = 1;
 	init_qp_attr.qp_type = IBV_QPT_RC;
-	init_qp_attr.send_cq = node->cq;
-	init_qp_attr.recv_cq = node->cq;
+	init_qp_attr.send_cq = node->cq[SEND_CQ_INDEX];
+	init_qp_attr.recv_cq = node->cq[RECV_CQ_INDEX];
 	ret = rdma_create_qp(node->cma_id, node->pd, &init_qp_attr);
 	if (ret) {
-		printf("cmatose: unable to create QP: %d\n", ret);
+		perror("cmatose: unable to create QP");
 		goto out;
 	}
 
@@ -227,12 +233,12 @@ static int addr_handler(struct cmatest_node *node)
 		ret = rdma_set_option(node->cma_id, RDMA_OPTION_ID,
 				      RDMA_OPTION_ID_TOS, &tos, sizeof tos);
 		if (ret)
-			printf("cmatose: set TOS option failed: %d\n", ret);
+			perror("cmatose: set TOS option failed");
 	}
 
 	ret = rdma_resolve_route(node->cma_id, 2000);
 	if (ret) {
-		printf("cmatose: resolve route failed: %d\n", ret);
+		perror("cmatose: resolve route failed");
 		connect_error();
 	}
 	return ret;
@@ -257,7 +263,7 @@ static int route_handler(struct cmatest_node *node)
 	conn_param.retry_count = 5;
 	ret = rdma_connect(node->cma_id, &conn_param);
 	if (ret) {
-		printf("cmatose: failure connecting: %d\n", ret);
+		perror("cmatose: failure connecting");
 		goto err;
 	}
 	return 0;
@@ -294,7 +300,7 @@ static int connect_handler(struct rdma_cm_id *cma_id)
 	conn_param.initiator_depth = 1;
 	ret = rdma_accept(node->cma_id, &conn_param);
 	if (ret) {
-		printf("cmatose: failure accepting: %d\n", ret);
+		perror("cmatose: failure accepting");
 		goto err2;
 	}
 	return 0;
@@ -356,8 +362,11 @@ static void destroy_node(struct cmatest_node *node)
 	if (node->cma_id->qp)
 		rdma_destroy_qp(node->cma_id);
 
-	if (node->cq)
-		ibv_destroy_cq(node->cq);
+	if (node->cq[SEND_CQ_INDEX])
+		ibv_destroy_cq(node->cq[SEND_CQ_INDEX]);
+
+	if (node->cq[RECV_CQ_INDEX])
+		ibv_destroy_cq(node->cq[RECV_CQ_INDEX]);
 
 	if (node->mem) {
 		ibv_dereg_mr(node->mr);
@@ -409,7 +418,7 @@ static void destroy_nodes(void)
 	free(test.nodes);
 }
 
-static int poll_cqs(void)
+static int poll_cqs(enum CQ_INDEX index)
 {
 	struct ibv_wc wc[8];
 	int done, i, ret;
@@ -419,7 +428,7 @@ static int poll_cqs(void)
 			continue;
 
 		for (done = 0; done < message_count; done += ret) {
-			ret = ibv_poll_cq(test.nodes[i].cq, 8, wc);
+			ret = ibv_poll_cq(test.nodes[i].cq[index], 8, wc);
 			if (ret < 0) {
 				printf("cmatose: failed polling CQ: %d\n", ret);
 				return ret;
@@ -440,8 +449,8 @@ static int connect_events(void)
 			cma_handler(event->id, event);
 			rdma_ack_cm_event(event);
 		} else {
-			printf("cmatose: failure in rdma_get_cm_event in connect events\n");
-			ret = err;
+			perror("cmatose: failure in rdma_get_cm_event in connect events");
+			ret = errno;
 		}
 	}
 
@@ -459,8 +468,8 @@ static int disconnect_events(void)
 			cma_handler(event->id, event);
 			rdma_ack_cm_event(event);
 		} else {
-			printf("cmatose: failure in rdma_get_cm_event in disconnect events\n");
-			ret = err;
+			perror("cmatose: failure in rdma_get_cm_event in disconnect events");
+			ret = errno;
 		}
 	}
 
@@ -476,7 +485,7 @@ static int migrate_channel(struct rdma_cm_id *listen_id)
 
 	channel = rdma_create_event_channel();
 	if (!channel) {
-		printf("cmatose: failed to create event channel\n");
+		perror("cmatose: failed to create event channel");
 		return -1;
 	}
 
@@ -491,7 +500,7 @@ static int migrate_channel(struct rdma_cm_id *listen_id)
 		rdma_destroy_event_channel(test.channel);
 		test.channel = channel;
 	} else
-		printf("cmatose: failure migrating to channel: %d\n", ret);
+		perror("cmatose: failure migrating to channel");
 
 	return ret;
 }
@@ -526,7 +535,7 @@ static int run_server(void)
 	printf("cmatose: starting server\n");
 	ret = rdma_create_id(test.channel, &listen_id, &test, RDMA_PS_TCP);
 	if (ret) {
-		printf("cmatose: listen request failed\n");
+		perror("cmatose: listen request failed");
 		return ret;
 	}
 
@@ -540,13 +549,13 @@ static int run_server(void)
 	test.src_in.sin_port = port;
 	ret = rdma_bind_addr(listen_id, test.src_addr);
 	if (ret) {
-		printf("cmatose: bind address failed: %d\n", ret);
+		perror("cmatose: bind address failed");
 		goto out;
 	}
 
 	ret = rdma_listen(listen_id, 0);
 	if (ret) {
-		printf("cmatose: failure trying to listen: %d\n", ret);
+		perror("cmatose: failure trying to listen");
 		goto out;
 	}
 
@@ -563,12 +572,12 @@ static int run_server(void)
 		}
 
 		printf("completing sends\n");
-		ret = poll_cqs();
+		ret = poll_cqs(SEND_CQ_INDEX);
 		if (ret)
 			goto out;
 
 		printf("receiving data transfers\n");
-		ret = poll_cqs();
+		ret = poll_cqs(RECV_CQ_INDEX);
 		if (ret)
 			goto out;
 		printf("data transfers complete\n");
@@ -622,7 +631,7 @@ static int run_client(void)
 					src_addr ? test.src_addr : NULL,
 					test.dst_addr, 2000);
 		if (ret) {
-			printf("cmatose: failure getting addr: %d\n", ret);
+			perror("cmatose: failure getting addr");
 			connect_error();
 			return ret;
 		}
@@ -634,7 +643,7 @@ static int run_client(void)
 
 	if (message_count) {
 		printf("receiving data transfers\n");
-		ret = poll_cqs();
+		ret = poll_cqs(RECV_CQ_INDEX);
 		if (ret)
 			goto disc;
 
